@@ -21,7 +21,9 @@
 
 import argparse
 import logging
+import os
 import sys
+from functools import wraps
 from http.client import HTTPConnection
 
 from .. import __version__
@@ -43,7 +45,7 @@ class HelpFormatter(
     def _format_usage(self, usage, actions, groups, prefix):
         if prefix is None:
             prefix = "Usage: "
-        return super(HelpFormatter, self)._format_usage(
+        return super()._format_usage(
             usage,
             actions,
             groups,
@@ -53,6 +55,7 @@ class HelpFormatter(
 
 class ArgumentParser(argparse.ArgumentParser):
     def __init__(self, *args, **kwargs):
+        add_auth = kwargs.pop("add_auth", True)
         add_helpers = kwargs.pop("add_helpers", True)
         version = kwargs.pop("version", __version__)
         manpage = kwargs.pop("manpage", None)
@@ -62,56 +65,104 @@ class ArgumentParser(argparse.ArgumentParser):
         super(ArgumentParser, self).__init__(*args, **kwargs)
 
         self._positionals.title = "Required arguments"
-        self._optionals.title = "Options"
+        if add_auth:
+            self._optionals.title = "Other options"
+        else:
+            self._optionals.title = "Options"
 
         # add manpage options for argparse-manpage
         self._manpage = manpage
 
+        # add auth options group
+        if add_auth:
+            self.add_auth_arguments()
+            group = self._action_groups.pop(-1)
+            self._action_groups.insert(1, group)
+
         # add helper commands group
         if add_helpers:
-            helpers = self.add_argument_group("Helper arguments")
-            helpers.add_argument(
-                "-h",
-                "--help",
-                action="help",
-                default=argparse.SUPPRESS,
-                help="show this help message and exit",
-            )
-            helpers.add_argument(
-                "-l",
-                "--list-idps",
-                action=ListIdpsAction,
-            )
-            if version is not None:
-                helpers.add_argument(
-                    "-V",
-                    "--version",
-                    action="version",
-                    version=version,
-                )
+            self.add_helper_arguments(version=version)
 
-    def add_identity_provider_argument(
+    def add_helper_arguments(self, version=__version__):
+        helpers = self.add_argument_group("Helper arguments")
+        helpers.add_argument(
+            "-h",
+            "--help",
+            action="help",
+            default=argparse.SUPPRESS,
+            help="show this help message and exit",
+        )
+        helpers.add_argument(
+            "-l",
+            "--list-idps",
+            action=ListIdpsAction,
+        )
+        if version is not None:
+            helpers.add_argument(
+                "-V",
+                "--version",
+                action="version",
+                version=version,
+            )
+        return helpers
+
+    def add_auth_arguments(
             self,
-            *args,
-            **kwargs
+            identity_provider=True,
+            kerberos=True,
+            username=True,
     ):
-        if not args:
-            args = ("-i", "--identity-provider")
-        kwargs.setdefault("default", _get_default_idp())
-        kwargs.setdefault(
-            "help",
-            "name of institution providing the identity (e.g. 'Cardiff "
-            "University'), or domain name of IdP host (e.g. idp.cf.ac.uk), "
-            "see --list-idps for a list of Identity Provider (IdPs) and "
-            "their IdP URL. "
-            "Shortened institution names (e.g. 'Cardiff') can be given as "
-            "long as they uniquely match a full institution name known by "
-            "CILogon",
-        )
-        return super(ArgumentParser, self).add_argument(
-            *args,
-            **kwargs
-        )
+        auth = self.add_argument_group("Authentication options")
+
+        if identity_provider:
+            defaultidp = _get_default_idp()
+            auth.add_argument(
+                "-i",
+                "--identity-provider",
+                default=defaultidp,
+                help="name of institution providing the identity (e.g. "
+                     "'Cardiff University'), or domain name of IdP host "
+                     "(e.g. idp.cf.ac.uk), see --list-idps for a list of "
+                     "Identity Provider (IdPs) and their IdP URL. Shortened "
+                     "institution names (e.g. 'Cardiff') can be given as "
+                     "long as they uniquely match a full institution name "
+                     "known by CILogon",
+            )
+
+        if kerberos:
+            auth.add_argument(
+                "-k",
+                "--kerberos",
+                action="store_true",
+                default=False,
+                help="enable kerberos negotiation",
+            )
+
+        if username:
+            auth.add_argument(
+                "-u",
+                "--username",
+                help="authentication username, will be prompted for if not "
+                     "given and not using --kerberos",
+            )
+
+        return auth
+
+    @wraps(argparse.ArgumentParser.parse_args)
+    def parse_args(self, *args, **kwargs):
+        args = super().parse_args(*args, **kwargs)
+        # if -X/--destroy wasn't given and
+        # -i/--identity-provider also wasn't given (and is supported)
+        # then raise an error
+        #    - this just supports giving -X/--destroy without having to
+        #      also give -i/--identity-provider for no reason
+        destroy = getattr(args, "destroy", None)
+        idp = getattr(args, "identity_provider", False)
+        if not destroy and idp is None:
+            self.error(
+                "the following arguments are required: -i/--identity-provider",
+            )
+        return args
 
 
 class ListIdpsAction(argparse.Action):
@@ -122,7 +173,7 @@ class ListIdpsAction(argparse.Action):
             default=argparse.SUPPRESS,
             help="show IdP names and URLs and exit",
     ):
-        super(ListIdpsAction, self).__init__(
+        super().__init__(
             option_strings=option_strings,
             dest=dest,
             default=default,
@@ -190,3 +241,22 @@ def init_logging(level=logging.DEBUG):
     requests_log.setLevel(level)
     requests_log.propagate = True
     return requests_log
+
+
+def destroy_file(path, desc=None, verbose=False):
+    """Destroy a file (if it exists), with verbose output
+    """
+    if verbose:
+        desc = (desc or "").rstrip(" ") + " "
+        print("Removing {}'{!s}'...".format(desc, path), end=" ")
+    try:
+        os.unlink(path)
+    except FileNotFoundError:  # file doesn't exit, no matter
+        if verbose:
+            print("not found", end="")
+    else:
+        if verbose:
+            print("done", end="")
+    finally:
+        if verbose:
+            print()
